
#include <stdio.h>
#include "smartcard.h"



/* 
 *   Defining a struct like this is a convenient way to organize the
 *      data you store on the smartcard's persistent memory.
 */
typedef struct {
	unsigned char canVote;   /* 1 if card allows voting, 0 otherwise */
} * dataOnCard;


void activateCard(void* cardData, void (*sendByte)(int), int (*recvByte)(void)){
	/*
         *  This gets called, on the smartcard, whenever the card is 
         *    inserted into any device.
         *
         *  <cardData> points to CardDataSize bytes of data that are the 
         *      persistent memory of the smartcard.  Any changes to this
         *      data will be written to the smartcard's persistent memory
         *      immediately after this function returns.
         *
         *  Calling sendByte(b) will send the byte b to the other device.
         *
         *  recvByte() will return the next byte sent by the other device.
         *
         *  The version you were given is insecure; you must fix this.
         */

	dataOnCard data = cardData;
	int        newval;

	/* send status to other device */
	sendByte(data->canVote);

	/* get new status value from other device */
	newval = recvByte();
	data->canVote = newval;
}
