
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include "spawnCard.h"

/*
 *  STUDENTS MAY NOT MODIFY THIS FILE.
 *
 *  This file contains code, to be invoked by the encoder and machine programs,
 *     that will invoke the smartcard program and set up communication channels
 *     to talk to it.
 */


static void closeOrDie(int fd){
	int e;

	e = close(fd);
	assert(!e);
}


void spawnCard(char cardFileName[], FILE** toCardFp, FILE** fromCardFp){
	int  toCardPipe[2];
	int  fromCardPipe[2];
	int  e;
	char outFdStr[32];
	char inFdStr[32];

	/* set up two pipes for communication to and from the smartcard */
	e = pipe(toCardPipe);
	assert(!e);
	e = pipe(fromCardPipe);
	assert(!e);
	
	/* create a new process and have it run the smartcard program */
	switch(fork()){
	case -1:
		/*  fork failed */
		fprintf(stderr, "ERROR: Couldn't fork card process.\n");
		exit(1);
		break;
	case 0:
		/*  we're the child (card) process  */

		/* close unused file descriptors */
		closeOrDie(toCardPipe[1]);
		closeOrDie(fromCardPipe[0]);

		/* set up strings with file descriptor values */
		sprintf(outFdStr, "%d", fromCardPipe[1]);
		sprintf(inFdStr, "%d", toCardPipe[0]);

		/* exec the smartcard program */
		execl("smartcard", "smartcard", 
		      cardFileName, inFdStr, outFdStr, 
		      (void*)0);

		/* should never get here */
		assert(0);
		break;
	default:
		/*  we're the parent process */

		/*  tell caller about the communication channels we set up */
		*toCardFp = fdopen(toCardPipe[1], "w");
		assert(*toCardFp);
		*fromCardFp = fdopen(fromCardPipe[0], "r");
		assert(*fromCardFp);

		/*  close the file descriptors we're not using */
		closeOrDie(toCardPipe[0]);
		closeOrDie(fromCardPipe[1]);

		break;
	}
}
